"""
This type stub file was generated by pyright.
"""

from collections.abc import Sequence
from typing import TYPE_CHECKING

"""License specification constants."""
if TYPE_CHECKING:
    ...
NOT_SUPPORTED: set[str]
SPDX_LICENSE_MAP: dict[str, Sequence[str]]
SPDX_LICENSE_EXCEPTIONS: tuple[str]



"""
This type stub file was generated by pyright.
"""

from collections.abc import Mapping
from dataclasses import Field
from dataclasses import dataclass
from typing import TYPE_CHECKING
from typing import Any
from typing import Callable
from typing import ClassVar
from typing import Iterator
from typing import Protocol
from typing import TypeAlias
from typing import TypeVar

"""Base dataclasses for OZI Metadata."""
if TYPE_CHECKING:
    VT = TypeVar('VT', str, int, float, bytes, None, tuple[str, ...])
    _Val: TypeAlias = list['_Key[VT]'] | Mapping['_Key[VT]', VT] | VT
    _Key: TypeAlias = VT | _Val[VT]
    _Lambda: TypeAlias = Callable[[], '_FactoryMethod']
    _FactoryMethod: TypeAlias = Callable[[], _Lambda]

class _FactoryDataclass(Protocol):
    """A dataclass that, when called, returns a factory method."""

    __dataclass_fields__: ClassVar[dict[str, Field[Any]]]
    def asdict(self) -> Mapping[str, _Val[str]]: ...
    def __call__(self) -> Field[_FactoryMethod]: ...
    def __iter__(self: _FactoryDataclass) -> Iterator[tuple[str, _Val[VT]]]: ...

def get_default(obj: _FactoryDataclass, name: str) -> _Val[VT] | Mapping[str, _Val[str]]:
    """Get a field from a Default by name.

    :param obj: a target object
    :type obj: _FactoryDataclass
    :param name: an attribute name
    :type name: str
    :return: attribute value
    :rtype: _Val[VT] | Mapping[str, _Val[str]]
    """

@dataclass(frozen=True, repr=False)
class Default(_FactoryDataclass):
    """A dataclass that, when called, returns it's own default factory field."""

    def __call__(self) -> Field[_FactoryMethod]:
        """Returns this dataclass as a Field.

        :return: A Field initialized with a factory method.
        :rtype: Field[_FactoryMethod]
        """
        ...

    def __iter__(self) -> Iterator[tuple[str, _Val[VT]]]:
        """Iterate through all fields.

        :yield: Fields as a tuple of name and values
        :rtype: Iterator[tuple[str, _Val[VT]]]
        """
        ...

    def asdict(self) -> Mapping[str, _Val[str]]:
        """Return a dictionary of all fields where repr=True.
        Hide a variable from the dict by setting repr to False and using
        a Default subclass as the default_factory.
        Typing is compatible with JSON and Jinja2 global namespace.

        .. seealso::

           :std:ref:`jinja2:global-namespace`
        """
        ...

    def __repr__(self) -> str:
        """Uses reprlib.repr with the default limits."""
        ...

    def __len__(self) -> int:
        """Get the total number of keys, including the class docstring."""
        ...



# ozi/spec/_spec.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Specification API for OZI Metadata."""
from dataclasses import dataclass
from dataclasses import field
from importlib.metadata import PackageNotFoundError
from importlib.metadata import version

from ozi_spec.base import Default
from ozi_spec.ci import RuffLint
from ozi_spec.project import ClassicProject
from ozi_spec.project import PythonProject
from ozi_spec.python import PythonSupport


def _current_version() -> str:
    """Returns the currently installed version of ozi-spec."""
    try:
        version_ = version('ozi-spec')
    except PackageNotFoundError:  # pragma: no cover
        version_ = '0.11'
    return version_


def _ozi_version() -> str:
    """Returns the current OZI release branch."""
    (
        major,
        minor,
        *_,
    ) = _current_version().split('.')
    if int(major) == 0:
        return f'{int(major) + 1}.{int(minor) + 13}'
    return f'{int(major) + 1}.{int(minor)}'  # pragma: defer to 1.0


@dataclass(slots=True, frozen=True, eq=True)
class Spec(Default):
    """OZI Specification metadata."""

    version: str = field(
        default_factory=_current_version,
        metadata={'help': 'OZI specification standard version.'},
    )
    python: PythonProject = ClassicProject()


@dataclass(slots=True, frozen=True, eq=True)
class Experimental(Default):
    """Experimental OZI specifications."""

    ruff: RuffLint = RuffLint()


@dataclass(slots=True, frozen=True, eq=True)
class OZI(Default):
    """OZI distribution metadata."""

    version: str = field(
        default_factory=_ozi_version,
        metadata={'help': 'Current release branch of the OZI package.'},
    )
    python_support: PythonSupport = PythonSupport()
    experimental: Experimental = Experimental()


@dataclass(slots=True, frozen=True, eq=True)
class Metadata(Default):
    """OZI metadata."""

    ozi: OZI = OZI()
    spec: Spec = field(default_factory=Spec)



# ozi/spec/project.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Project specification metadata."""
from dataclasses import dataclass

from ozi_spec.base import Default
from ozi_spec.ci import CI
from ozi_spec.ci import Build
from ozi_spec.ci import CheckpointSuite
from ozi_spec.ci import ClassicDist
from ozi_spec.ci import ClassicLint
from ozi_spec.ci import ClassicTest
from ozi_spec.ci import RuffLint
from ozi_spec.pkg import Pkg
from ozi_spec.python import Support
from ozi_spec.src import Src

@dataclass(slots=True, frozen=True, eq=True, repr=False)
class PythonProject(Default):
    """Base class for Python Project specification metadata."""

    ci: CI = CI()
    support: Support = Support()
    dist: ClassicDist = ClassicDist()
    lint: ClassicLint = ClassicLint()
    test: ClassicTest = ClassicTest()
    build: Build = Build()
    pkg: Pkg = Pkg()
    src: Src = Src()

@dataclass(slots=True, frozen=True, eq=True, repr=False)
class ClassicProject(PythonProject):
    """OZI project using classic Python checkpoint toolchains."""

@dataclass(slots=True, frozen=True, eq=True, repr=False)
class RuffProject(PythonProject):
    """Alternative to classic project using ruff for linting and formatting."""

    lint: ClassicLint = ClassicLint(
        exclude=('meson-private',),
        module=('ruff', 'mypy', 'pyright'),
        plugin={},
        utility={'ruff': 'ruff>=0.1.6', 'mypy': 'mypy', 'pyright': 'pyright'},
        ignore=(
            'A003',
            'ARG',
            'ANN401',
            'TRY003',
            'B028',
            'B905',
            'D1',
            'D2',
            'D101',
            'D4',
            'FLY',
            'FBT',
            'PGH003',
            'PLR',
            'RET',
            'EM',
            'PLW',
            'PTH',
            'RUF009',
            'RUF012',
            'RUF015',
            'RUF200',
            'SIM',
            'T201',
            'TCH002',
            'TCH004',
            'UP',
            'PERF203',
        ),
    )
